# **Service Management and Control Decisions**

© University of Southampton IT Innovation Centre, 2018

This document describe possible approaches for integrating FLIPS monitoring with the CLMC. 

## **Authors**

|Authors|Organisation|                    
|-|-|
|[Michael Boniface](mailto:mjb@it-innovation.soton.ac.uk)|[University of Southampton, IT Innovation Centre](http://www.it-innovation.soton.ac.uk)|
|[Simon Crowle](mailto:sgc@it-innovation.soton.ac.uk)|[University of Southampton, IT Innovation Centre](http://www.it-innovation.soton.ac.uk)|

### Integration with FLIPS Monitoring

FLIPS offers a scalable pub/sub system for distributing monitoring data. The architecture is described in the POINT monitoring specification https://drive.google.com/file/d/0B0ig-Rw0sniLMDN2bmhkaGIydzA/view. Some observations can be made

* MOOSE and CLMC provide similar functions in the architecture, the CLMC will not have access to MOOSE but will need to subscribe to data points provided by FLIPS
* The APIs for Moly and Blackadder are not provided therefore it's not possible to critically understand the correct implementation approach for agents and monitoring data distribution
* Individual datapoints need to be aggregated into measurements according to a sample rate
* We may need to use the blackadder API for distribution of monitoring data, replacing messaging systems such as AMQP with all buffering and pub/sub deployed on the nodes themselves rather than a central service. 

There are a few architectural choices. The first below uses moly as an integration point for monitoring processes via a Telegraf output plugin with data inserted into influx using a blackadder API input plugin on another Telegraf agent running on the CLMC. In this case managing the subscriptions to nodes and data points is difficult. In addition, some data points will be individual from FLIPS monitoring whilst others will be in line protocol format from Telegraf. For the FLIPS data points a new input plugin would be required to aggregate individual data points into time-series measurements. 

![FLIPSAgentArchitecture](/docs/image/flips-monitoring-architecture.jpg)

The second (currently preferred) choice only sends line protocol format over the wire. Here we develop telegraf input and output plugins for blackadder benefiting from the scalable nature of the pub/sub system rather than introducing RabbitMQ as a central server. In this case the agent on each node would be configured with input plugins for service, host and network . We'd deploy a new Telegraf input plugin for FLIPS data points on the node's agent by subscribing to blackadder locally and then publish the aggregated measurement using the line protocol back over blackadder to the CLMC. FLIPS can still publish data to MOOSE as required. 

![FLIPSAgentArchitecture](/docs/image/flips-monitoring-architecture2.jpg)

The pub/sub protocol still needs some work as we don't want the CLMC to have to subscribe to nodes as they start and stop. We want the nodes to register with a known CLMC and then start publishing data to the CLMC according to a monitoring configuration (e.g. sample rate, etc). So we want a "monitoring topic" that nodes publish to and that the CLMC can pull data from. This topic is on the CLMC itself and note the nodes. Reading the FLIPS specification it seems that this is not how the nodes current distribute data, although could be wrong



#### Network Measurements

**net_port_config**

network config is concerned with any network io allocation/constraints for network rx/tx. Possible fields (but these are not available from the FLIPS monitoring specification)

`net_port_config,<common_tags>,port_id,port_state RX_USAGE_CONSTRAINT, TX_USAGE_CONSTRAINT, RX_THROUGHPUT_CONSTRAINT, TX_THROUGHPUT_CONSTRAINT timestamp`

Specific tags
* port_state
* port_id

**net_port_io**

All net_port_io measurements are monitoring by FLIPS. Note that RX_PACKETS_M seems to have inconsistent naming convention unless we are mistaken

`net_port_io,<common_tags>,port_id PACKET_DROP_RATE_M, PACKET_ERROR_RATE_M, RX_PACKETS_M, TX_PACKETS_PORT_M, RX_BYTES_PORT_M, TX_BYTES_PORT_M  timestamp`

Specific tags
* port_id 

**Worked Usage Scenario - MPEG-DASH**

The  scenario aims to verify two aspects

* CLMC monitoring specification & data acquisition
* Support for initial decision making processes for FLAME (re)orchestration

The scenario is being developed in further document here:

https://gitlab.it-innovation.soton.ac.uk/mjb/flame-clmc/blob/integration/docs/CLMC%20monitoring%20specification%20for%20a%20basic%20scenario.md

The FLAME platform acquires a slice of the infrastructure resources (compute, RAM & storage [C1, C2, C3] and networking). A media service provider offers an MPEG-DASH service to end-users (via their video clients connected to NAPs on the FLAME platform). The service provider deploys surrogates of the MPEG-DASH service on all compute nodes [C1-C3]. All services (including NAPs) are monitored by the CLMC.

Over time a growing number of video clients use a MPEG-DASH service to stream movies on demand. As clients connect and make requests, the platform makes decisions and takes actions in order to maintain quality of service for the increasing number of clients demanding an MPEG-DASH service.

What are the possible criteria (based on metrics and analytics provided by the CLMC) that could be used to help NAP makes these decisions?

In this scenario what are the possible actions a NAP could take?

![Scenario](/docs/image/scenario.jpg)

![Scenario Measurements](/docs/image/scenario-measurements.jpg)

Platform actions

* Increase the resources available to MPEG-DASH surrogates

 * This may not be possible if resources unavailable
 * Vertical scaling may not solve the problem (i.e., I/O bottleneck)

* Re-route client requests to other MPEG-DASH services

 * C1 – Closer to clients, but limited capability
 * C3 – Greater capability but further away from clients
… note: NAP service end-point re-routing will need to take into account network factors AND compute resource availability related service KPIs; i.e., end-to-end performance

Service actions

* Lower overall service quality to clients… reduce overall resource usage

* Avg quality met: the ratio of average delivered quality out of requested quality
* Avg start up time: the average time taken before a video stream starts playing less than a threshold
* Avg video stalls: the percentage of stalls (dropped video segments that require re-sending) less than a threshold

# **MISC Measurements and Further Questions**

The following data points require further analysis

* CPU_UTILISATION_M: likely to be replaced by other metrics provided directly by Telegraf plugins
* END_TO_END_LATENCY_M: not clear what this measurement means, so needs clarification
* BUFFER_SIZES_M: needs clarification 
* RX_PACKETS_IP_M: is this just NAP or all Nodes
* TX_PACKETS_IP_M: is this just NAP or all Nodes

The following fields need further analysis as they seem to relate to core ICN, most likely fields/measurements related to platform components

* FILE_DESCRIPTORS_TYPE_M 
* MATCHES_NAMESPACE_M
* PATH_CALCULATIONS_NAMESPACE_M 
* PUBLISHERS_NAMESPACE_M 
* SUBSCRIBERS_NAMESPACE_M 

The following fields relate to CID which I don't understand but jitter is an important metric so we need to find out.

* PACKET_JITTER_CID_M
* RX_BYTES_CID_M 
* TX_BYTES_CID_M 

Some questions

* Can a single value of jitter (e.g. avg jitter) be calculated from the set of measurements in PACKET_JITTER_CID_M message? What is the time period for the list of jitter measurements?
* What does CID  mean? consecutive identical digits


__Can we measure network usage for a specific VM from FLIPS monitoring?__
__Some metrics from FLIPS contain 'port' label, others not, is this intended?__


QUESTIONS
1. Is the content navigation tag and fully qualified domain name (SDN based)? [Most likely: yes] although this may only be part of the URL?

#### Link Measurements

Links are established between VM/container instances, need to discuss what measurements make sense. Also the context for links could be between media services, therefore a link measurement should be within the platform context and NOT the media service context. Need a couple of scenarios to work this one out. 

**link_config**

Link Tags

* link_name
* link_id
* source_node_id
* destination_node_id
* link_type
* link_state

**link_perf**

link perf is measured at the nodes, related to end_to_end_latency. Needs further work.

# Other Issues

**Trust in measurements**

If the agent is deployed in a VM/container that a tenant has root access then a tenant could change the configuration to fake measurements associated with network and host in an attempt gain benefit. This is a security risk. Some ideas include

* Deploy additional agents on hosts rather than agents to measure network and VM performance. Could be hard to differentiate between the different SFs deployed on a host
* Generate a hash from the agent configuration file that's checked within the monitoring message. Probably too costly and not part of the telegraf protocol
* Use unix permissions (e.g. surrogates are deployed within root access to them)